<?php

namespace App\Http\Controllers\Auth;

use App\LGA;
use App\PollingUnit;
use App\Pus;
use App\State;
use App\User;
use App\Http\Controllers\Controller;
use App\Http\Utils\utils;
use App\Ward;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Foundation\Auth\RegistersUsers;


class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = '/home';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'last_name' => 'required|string|max:255',
            'first_name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6',
            'state' => 'required|integer',
            'phone' => 'required|string',
            'lga' => 'required|integer',
            'gender' => 'required|string',
            'dob' => 'required|string',
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array $data
     * @return \App\User
     */
    protected function create(array $data)
    {

        $user = User::create([
            'first_name' => $data['first_name'],
            'last_name' => $data['last_name'],
            'email' => $data['email'],
            'state_id' => $data['state'],
            'lga_id' => $data['lga'],
            'gender' => $data['gender'],
            'phone' => $data['phone'],
            'reg_type' => 'normal',
            'dob' => '1900-01-01',
            'password' => Hash::make($data['password']),

        ]);
    }

    protected function createNormal(Request $request)
    {
        $validated = $this->validator($request->all());
        if ($validated->fails()) {

            return utils::makeResponse([], $validated->errors()->getMessages(), false);
        }
        try {
            $userinfo = [
                'first_name' => $request->first_name,
                'last_name' => $request->last_name,
                'gender' => strtolower($request->gender),
                'email' => $request->email,
                'password' => Hash::make($request->password),
                'state_id' => $request->state,
                'reg_type' => 'normal',
                'dob' => $request->dob,
                'phone' => $request->phone,
                'lga_id' => $request->lga,
                'pu_id' => 0,
                'reg_time' => date('Y-m-d'),
            ];
            $user = User::create($userinfo);
            $uid = $user->id;
            $pu = new PollingUnit();
            $pu->lga_id = $request->lga;
            $pu->user_id = $uid;
            $pu->state_id = $request->state;
            $pu->save();
            $user->pu_id = $pu->id;
            $user->save();
            return utils::makeResponse(['token' => utils::generateToken($user->id)]);
        } catch (\Exception $e) {
            return utils::makeResponse([], __($e->getMessage()), false);
        }

    }

    protected function createVin(Request $request)
    {
        $validated = Validator::make($request->all(), [
            'last_name' => 'required|string|max:255',
            'vin' => 'required|string|min:5',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6',
            'state' => 'required|integer',
            'phone' => 'required|string'
        ]);
        if ($validated->fails()) {
            return utils::makeResponse([], $validated->errors()->getMessages(), false);
        }
        $client = new Client();
        $url = "https://vaultnoir.com/api/check_vin.php?vin=$request->vin&state=$request->state&last_name=$request->last_name";
        try {
            $result = $client->get($url);
            if ($result->getBody()) {
                $response = $result->getBody()->read(1024);
                $data = json_decode($response);
                $pudata = explode('/', $data->polling_unit_code);
                $lga = LGA::where(['lga_code' => $pudata[1], 'state_id' => $request->state])->first();
                $lga_id = $lga->id;
                $pus = Pus::where(['ward_code' => $pudata[2], 'state_id' => $request->state, 'lga_id' => $lga_id, 'poll_code' => $pudata[3]])->first();
                $userinfo = [
                    'vin' => $data->vin,
                    'first_name' => $data->first_name,
                    'last_name' => $data->last_name,
                    'gender' => strtolower($data->gender),
                    'email' => $request->email,
                    'password' => Hash::make($request->password),
                    'state_id' => $request->state,
                    'reg_type' => 'vin',
                    'dob' => '1900-01-01',
                    'phone' => $request->phone,
                    'pu_id' => $pus->id,
                    'reg_time' => date('Y-m-d'),
                    'lga_id' => $lga_id,
                ];
                $user = User::create($userinfo);
                return utils::makeResponse(['token' => utils::generateToken($user->id)]);
            }

        } catch (\Exception $e) {
            return utils::makeResponse([], __($e->getMessage()), false);
        }
    }

}
